#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
from datetime import datetime

# import interfaces
from pyams_workflow.interfaces import IWorkflowPublicationInfo, IWorkflowState, IWorkflowVersions, IWorkflowInfo, \
    ObjectClonedEvent, IWorkflow

# import packages
from pyams_utils.registry import utility_config
from pyams_workflow.workflow import Transition, Workflow
from pyramid.threadlocal import get_current_registry
from zope.copy import copy
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm

from pyams_portal import _


DRAFT = 'draft'  # new template
PUBLISHED = 'published'  # published template
RETIRED = 'retired'  # retired template
ARCHIVED = 'archived'  # archive done
DELETED = 'deleted'  # deleted template


STATUS_IDS = ('draft', 'published', 'retired', 'archived', 'deleted')
STATUS_LABELS = (_("Draft"),
                 _("Published"),
                 _("Retired"),
                 _("Archived"),
                 _("Deleted"))

STATUS_VOCABULARY = SimpleVocabulary([SimpleTerm(STATUS_IDS[i], STATUS_IDS[i], t)
                                      for i, t in enumerate(STATUS_LABELS)])


def publish_action(wf, context):
    """Publish template"""
    now = datetime.utcnow()
    wf_content = IWorkflowPublicationInfo(context)
    if wf_content.first_publication_date is None:
        wf_content.first_publication_date = now
    IWorkflowPublicationInfo(context).publication_date = now
    version_id = IWorkflowState(context).version_id
    for version in IWorkflowVersions(context).get_versions(('published', 'retired')):
        if version is not context:
            IWorkflowInfo(version).fire_transition_toward('archived',
                                                          comment="Published version {0}".format(version_id))


def clone_action(wf, context):
    """Duplicate template"""
    result = copy(context)
    registry = get_current_registry()
    registry.notify(ObjectClonedEvent(result, context))
    return result


def retire_action(wf, context):
    """Archive template"""
    now = datetime.utcnow()
    IWorkflowPublicationInfo(context).publication_expiration_date = now


def archive_action(wf, context):
    """Archive template"""
    now = datetime.utcnow()
    content = IWorkflowPublicationInfo(context)
    content.publication_expiration_date = min(content.publication_expiration_date or now, now)


def can_delete(wf, context):
    content = IWorkflowPublicationInfo(context)
    return content.publication_effective_date is None


def delete_action(wf, context):
    """Delete draft version"""
    parent = context.__parent__
    name = context.__name__
    del parent[name]


#
# Workflow transitions
#

init = Transition(transition_id='init',
                  title=_("Initialize"),
                  source=None,
                  destination=DRAFT)

draft_to_published = Transition('draft_to_published',
                                title=_("Publish..."),
                                source=DRAFT,
                                destination=PUBLISHED,
                                permission='portal.templates.manage',
                                action=publish_action,
                                order=1,
                                menu_css_class='fa fa-fw fa-play',
                                view_name='wf-publish.html',
                                html_help=_('''This content is currently in DRAFT mode.
                                               Publishing it will make it publicly visible.'''))

published_to_retired = Transition('published_to_retired',
                                  title=_("Retire..."),
                                  source=PUBLISHED,
                                  destination=RETIRED,
                                  permission='portal.templates.manage',
                                  action=retire_action,
                                  order=2,
                                  menu_css_class='fa fa-fw fa-stop',
                                  view_name='wf-retire.html',
                                  html_help=_('''This content is actually published.
                                                 You can retire it to make it invisible, but contents using this
                                                 template won't be visible anymore!'''))

published_to_draft = Transition('published_to_draft',
                                title=_("Create new version..."),
                                source=PUBLISHED,
                                destination=DRAFT,
                                permission='portal.templates.manage',
                                action=clone_action,
                                order=99,
                                menu_css_class='fa fa-fw fa-copy',
                                view_name='wf-clone.html')

retired_to_published = Transition('retired_to_published',
                                  title=_("Re-publish..."),
                                  source=RETIRED,
                                  destination=PUBLISHED,
                                  permission='portal.templates.manage',
                                  action=publish_action,
                                  order=1,
                                  menu_css_class='fa fa-fw fa-play',
                                  view_name='wf-publish.html',
                                  html_help=_('''This content was published and retired.
                                                 You can re-publish it to make it visible again.'''))

published_to_archived = Transition('published_to_archived',
                                   title=_("Archive..."),
                                   source=PUBLISHED,
                                   destination=ARCHIVED,
                                   permission='portal.templates.manage',
                                   action=archive_action,
                                   order=3,
                                   menu_css_class='fa fa-fw fa-archive',
                                   view_name='wf-archive.html',
                                   html_help=_('''This content is currently published.
                                                  If it is archived, it will not be possible to make it visible again
                                                  except by creating a new version!'''))

retired_to_archived = Transition('retired_to_archived',
                                 title=_("Archive..."),
                                 source=RETIRED,
                                 destination=ARCHIVED,
                                 permission='portal.templates.manage',
                                 action=archive_action,
                                 order=3,
                                 menu_css_class='fa fa-fw fa-archive',
                                 view_name='wf-archive.html',
                                 html_help=_('''This content has been published but is currently retired.
                                                If it is archived, it will not be possible to make it visible again
                                                except by creating a new version!'''))

archived_to_draft = Transition('archived_to_draft',
                               title=_("Create new version..."),
                               source=ARCHIVED,
                               destination=DRAFT,
                               permission='portal.templates.manage',
                               action=clone_action,
                               order=99,
                               menu_css_class='fa fa-fw fa-copy',
                               view_name='wf-clone.html')

deleted = Transition('delete',
                     title=_("Delete..."),
                     source=DRAFT,
                     destination=DELETED,
                     condition=can_delete,
                     action=delete_action,
                     order=6,
                     menu_css_class='fa fa-fw fa-trash',
                     view_name='wf-delete.html',
                     html_help=_('''This content has never been published.
                                    It can be removed and definitely deleted.'''))

wf_transitions = [init,
                  draft_to_published,
                  published_to_retired,
                  published_to_draft,
                  retired_to_published,
                  published_to_archived,
                  retired_to_archived,
                  archived_to_draft,
                  deleted]


wf = Workflow(wf_transitions,
              states=STATUS_VOCABULARY,
              published_states=(PUBLISHED,))


@utility_config(name='PyAMS portal template workflow', provides=IWorkflow)
class WorkflowUtility(object):
    """Workflow utility registration"""

    def __new__(cls):
        return wf
